-------------------------------------------------------------------------------
-- Entity:	EmifB
--
-- Project:	Uni1p
--
-- Author:	Antonin Hegar
--
-- Description:
--	Synchronous interface between DSP and FPGA - address space 2.
--		SYNCRL = 3
--		SYNCWR = 0
--		CEEXT = 0
--		RENEN = 1
--	Boot code for DSP is included - when reading from CE(1) space.
--	Register for control the DSP (reset, NMI) is included - it's located on address 0
--		of CE(0) space.
--
-- Revision:
--	$Log: EmifB.vhd,v $
--	Revision 1.3  2006/04/12 05:29:43  Tony
--	Improve timing - EMIFB interface use registers in IOBs.
--	
--	Revision 1.2  2005/11/02 21:20:05  Tony
--	Setup EMIFB interface (space CE2 - SSRAM) during boot of the DSP.
--	
--	Revision 1.1  2005/05/03 17:35:27  Tony
--	Initial release.
--	
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- LIBRARY
-------------------------------------------------------------------------------
library ieee;
library unisim;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.DX64Header.all;
use unisim.vcomponents.all;

-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
entity EmifB is
	generic (
    CLKIN_FREQ_C    : real := real(INPUT_CLOCK_FREQ_C);                       -- CLKIN frequency
    EMIF_FREQ_C		: real := real(FPGA_CLOCK_FREQ_C);						-- BECLKOUT frequency
		N_DEVICES_C		: natural := 3;							-- number of devices
		              														-- connected to local bus
		DCM_MULTIPLY_C	: natural := 2;							-- CLKFX_MULTIPLY
		DCM_DIVIDE_C	: natural := 1							-- CLKFX_DIVIDE
	);
	port (
		-----------------------------------------------------------------------
		-- global signals
--		NReset			: inout std_logic;                     	-- external asynchronous reset
		NReset_io_in			: in std_logic;                     	-- external asynchronous reset
		NReset_io_out			: out std_logic;                     	-- external asynchronous reset
		NReset_io_tri			: out std_logic;                     	-- external asynchronous reset
		
		ClockExt		: in std_logic;							-- external clock signal

   Debug : out std_logic_vector(7 downto 0);

        -----------------------------------------------------------------------
		-- DSP interface
		NDReset			: out std_logic;
		NMI				: out std_logic;

		BEA				: in std_logic_vector(8 downto 1);
--		BED				: inout std_logic_vector(15 downto 0);
		BED_io_in				: in std_logic_vector(15 downto 0);
		BED_io_out			: out std_logic_vector(15 downto 0);
		BED_io_tri			: out std_logic_vector(15 downto 0);

--		BBE				: inout std_logic_vector(1 downto 0);
-- 		BBE_io_in				: in std_logic_vector(1 downto 0);
-- 		BBE_io_out			: out std_logic_vector(1 downto 0);
-- 		BBE_io_tri			: out std_logic_vector(1 downto 0);
		
		BCE				: in std_logic_vector(3 downto 0);
		BARE			: in std_logic;
		BAWE			: in std_logic;
		BAOE			: in std_logic;
		BSO3			: in std_logic;
		BARDY			: out std_logic;
		BPDT			: in std_logic;
		BECLKIN			: out std_logic;
		BECLKOUT		: in std_logic;

        -----------------------------------------------------------------------
		-- internal bus
		AReset			: out std_logic;						-- async. reset of internal logic in FPGA
-- 		ClockInt		: out std_logic;						-- clock domain for DspBusArr
 		ClockInt		: in std_logic;						-- clock domain for DspBusArr
 		clk_ext      : out std_logic;
 		clk_dcm      : out std_logic;
 		clk_sel      : out std_logic;
-- 		I0 => clock_ext,
-- 	  I1 => dcm_clk0,
-- 	  S  => sel_beclkout,
-- 	  O  => clock_in
		
		
		ClockFx			: out std_logic;						-- Clock * DCM_MULTIPLY_C / DCM_DIVIDE_C
		DspBusArr		: inout DspBusArr_T(0 to N_DEVICES_C-1)
	);
end entity EmifB;


-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
architecture RTL of EmifB is

    ---------------------------------------------------------------------------
    -- attributes declaration
	attribute box_type		: string;
    attribute iob           : string;
    
	-- Digital Clock Manager
	component DCM is
	generic (
   	    CLKIN_PERIOD 		: real := 0.0;
    	DFS_FREQUENCY_MODE 	: string := "LOW";
    	DLL_FREQUENCY_MODE 	: string := "LOW";
    	DSS_MODE 			: string := "NONE";
    	DUTY_CYCLE_CORRECTION : boolean := true;
    	PHASE_SHIFT 		: integer := 0;
    	CLKDV_DIVIDE 		: real := 2.0;
		CLKIN_DIVIDE_BY_2 	: boolean := false;
    	CLKOUT_PHASE_SHIFT 	: string := "NONE";
	   	CLKFX_DIVIDE 		: integer := 1;
    	CLKFX_MULTIPLY 		: integer := 4;
    	CLK_FEEDBACK 		: string := "1X";
    	STARTUP_WAIT 		: boolean := false
    );
	port (
	    CLK0 		: out std_ulogic := '0';
	    CLK180 		: out std_ulogic := '0';
	    CLK270 		: out std_ulogic := '0';
	    CLK2X 		: out std_ulogic := '0';
	    CLK2X180 	: out std_ulogic := '0';
	    CLK90 		: out std_ulogic := '0';
	    CLKDV 		: out std_ulogic := '0';
	    CLKFX 		: out std_ulogic := '0';
	    CLKFX180 	: out std_ulogic := '0';
	    LOCKED 		: out std_ulogic := '0';
	    PSDONE 		: out std_ulogic := '0';
	    STATUS 		: out std_logic_vector(7 downto 0) := "00000000";

	    CLKFB 		: in std_ulogic := '0';
	    CLKIN 		: in std_ulogic := '0';
	    DSSEN 		: in std_ulogic := '0';
	    PSCLK 		: in std_ulogic := '0';
	    PSEN 		: in std_ulogic := '0';
	    PSINCDEC 	: in std_ulogic := '0';
	    RST 		: in std_ulogic := '0'
 	);
	end component DCM;
	attribute box_type of DCM : component is "black_box";

    ---------------------------------------------------------------------------
    -- IOBUF
    component IOBUF
        generic (
            DRIVE            : integer;
            IOSTANDARD       : string;
            SLEW             : string);
        port (
            O  : out   std_ulogic;
            IO : inout std_ulogic;
            I  : in    std_ulogic;
            T  : in    std_ulogic);
    end component;
	attribute box_type of IOBUF : component is "black_box";
    
	---------------------------------------------------------------------------
	-- IBUFG
	component IBUFG
	port(
		O : out std_ulogic;
		I : in std_ulogic);
	end component;
	attribute box_type of IBUFG : component is "black_box";

	---------------------------------------------------------------------------
	-- BUFG
	component BUFG
	port(
		O : out std_ulogic;
		I : in std_ulogic);
	end component;
   	attribute box_type of BUFG : component is "black_box";

  	---------------------------------------------------------------------------
	-- BUFGMUX
	component BUFGMUX
    port(
      	O  : out std_ulogic;
      	I0 : in  std_ulogic;
      	I1 : in  std_ulogic;
      	S  : in  std_ulogic);
  	end component;
    attribute box_type of BUFGMUX : component is "black_box";

    ---------------------------------------------------------------------------
    -- Calculate MULTIPLY_C and DIVIDE_C for DCM
    -- CLK_IN       input clock frequency
    -- CLK_OUT      requested output clock frequency
    -- Deviation    possible deviation from requested output frequency
    ---------------------------------------------------------------------------
    function CalcMultDiv( CLK_IN : real; CLK_OUT : real; Deviation : real; mult : boolean ) return natural is
        variable m, d : natural;
        variable t : real;
    begin
        m := 0;
        d := 0;
        for j in 1 to 32 loop
            for i in 2 to 32 loop
                t := CLK_IN * real(i) / real(j);
                if m = 0 and (t - Deviation) <= CLK_OUT and CLK_OUT <= (t + Deviation) then
                    m := i;
                    d := j;
                end if;
            end loop;   -- i
        end loop;   -- j
        if mult then
            return m;
        else
            return d;
        end if;
    end CalcMultDiv;
    
	---------------------------------------------------------------------------
	-- Type definition and constants
	---------------------------------------------------------------------------
	-- boot code length in bytes
	constant BOOT_CODE_LEN_C: natural := 16;
	-- type of boot code
	type BootCode_T is array (0 to BOOT_CODE_LEN_C-1) of unsigned(31 downto 0);
	-- boot instruction codes
	constant BOOT_CODE_C	: BootCode_T :=
								(
									X"0200082A", 			-- mvkl  EMIFB_CECTL2,B4
      								X"0200D46A",            -- mvkh  EMIFB_CECTL2,B4
      								X"02FFDFA8",            -- mvkl  0xffffffbf,A5
      								X"02FFFFE8",            -- mvkh  0xffffffbf,A5
      								X"029002F4",            -- stw   A5,*B4
     								X"0200282A",            -- mvkl  EMIFB_CESEC2,B4
									X"0200D46A",            -- mvkh  EMIFB_CESEC2,B4
      								X"028011A8",            -- mvkl  0x23,A5
      								X"02800068",            -- mvkh  0x23,A5
      								X"029002F4",            -- stw   A5,*B4

									X"0200782A",
									X"0230006A",
									X"029002B4",
									
									X"0080012A",			-- MVK 2h, B1
									X"020403A2",			-- MVC B1, IER
--									X"0001E000",			-- IDLE
--									X"0001E000",			-- IDLE
--									X"0001E000",			-- IDLE
									X"0001E000");			-- IDLE

--									X"02007FAA",            -- mvkl  0x600000FF,B4
--									X"0230006A",            -- mvkh  0x600000FF,B4
--      								X"029002B4",            -- stb   A5,*B4
--									X"0080012A",			-- MVK 2h, B1
--									X"020403A2",			-- MVC B1, IER
--									X"0001E000");			-- IDLE								(
	signal boot_code		: unsigned(7 downto 0);

	---------------------------------------------------------------------------
	-- Started up 
	---------------------------------------------------------------------------
	-- width of startup sequency
	constant STARTUP_COUNT_C 	: natural := 16;
	constant STARTUP_SETUP_C 	: natural := 4;
	constant STARTUP_HOLD_C		: natural := 4;
	signal startup_cnt 			: natural range 0 to STARTUP_COUNT_C-1 := 0;
	signal startup_seq  		: std_logic := '0';
	signal startup_pulse 		: std_logic := '0';
	signal startup_drive 		: std_logic := '0';
	
	---------------------------------------------------------------------------
	-- Signal definition
	---------------------------------------------------------------------------
	signal areset_in		: std_logic;
	signal clock_in			: std_logic;

	signal dreset_cnt		: natural range 0 to DRESET_PULSE_LEN_C := 0;
	signal dreset_done		: std_logic := '0';
	signal nmi_cnt			: natural range 0 to NMI_PULSE_LEN_C := 0;
	signal nmi_done			: std_logic := '0';


	signal dcm_in			: std_logic;
	signal dcm_clk0			: std_logic;
	signal dcm_clkfx		: std_logic;
	signal dcm_locked		: std_logic;
	signal dcm_reset		: std_logic;
	signal clock_ext		: std_logic;
	signal clock_emif		: std_logic;
	signal sel_beclkout		: std_logic;

    ---------------------------------------------------------------------------
    -- BECLKIN DCM
    ---------------------------------------------------------------------------
    constant CLKIN_MULT_C   : natural := CalcMultDiv( CLKIN_FREQ_C, EMIF_FREQ_C, 500.0e3, true);
    constant CLKIN_DIV_C    : natural := CalcMultDiv( CLKIN_FREQ_C, EMIF_FREQ_C, 500.0e3, false);
    signal dcm_in_clk0      : std_logic;
    signal dcm_in_clkfx     : std_logic;
    signal dcm_in_locked    : std_logic;
    signal dcm_in_reset     : std_logic;
    
	-- EMIFB interface
	signal read_in			: std_logic;
	signal write_in			: std_logic;
	signal read_data_valid	: std_logic;
	signal addr_in			: unsigned(DSP_ADDR_WIDTH_C downto 1);
	signal data_in			: unsigned(DSP_DATA_WIDTH_C-1 downto 0);
	signal data_out			: unsigned(DSP_DATA_WIDTH_C-1 downto 0);
-- 	signal bare_in			: std_logic;
	signal bawe_in			: std_logic;
	signal baoe_in			: std_logic;
	signal bce0_in			: std_logic;
	signal bce1_in			: std_logic;
    signal bed_tri          : unsigned(DSP_DATA_WIDTH_C-1 downto 0) := (others => '1');
    signal bed_out          : unsigned(DSP_DATA_WIDTH_C-1 downto 0) := (others => '1');
    signal bed_in           : unsigned(DSP_DATA_WIDTH_C-1 downto 0);
	
	constant RDY_DELAY_C	: natural := 3;
	signal rdy_delay		: unsigned(0 to RDY_DELAY_C);
	constant ARST_DELAY_C	: natural := 3;
	signal arst_delay		: unsigned(0 to ARST_DELAY_C);
	signal nreset_in 		: std_logic;
	signal nreset_out 		: std_logic;

	-- control register
	signal ctrl				: std_logic_vector(DX64_DATA_WIDTH_C-1 downto 0);
	constant CTRL_DRESETi	: natural := 0;
	constant CTRL_NMIi		: natural := 1;
	constant CTRL_LEDi		: natural := 2;


    ---------------------------------------------------------------------------
    -- attribute assignment to signals
    ---------------------------------------------------------------------------
    attribute iob of addr_in    : signal is "true";
    attribute iob of data_in    : signal is "true";
    attribute iob of bed_out    : signal is "true";
    attribute iob of bed_tri    : signal is "true";
    attribute iob of baoe_in    : signal is "true";
    attribute iob of bawe_in    : signal is "true";
    attribute iob of bce0_in    : signal is "true";
    attribute iob of bce1_in    : signal is "true";

	attribute pullup  			: string;
-- 	attribute pullup of NReset 	: signal is "true";
	
begin

    ---------------------------------------------------------------------------
    -- GLOBAL ASSERTION
    ---------------------------------------------------------------------------
    assert 2 <= CLKIN_MULT_C and CLKIN_MULT_C <= 32
                report "Can't find values for DCM MULTIPLY and DIVIDE parameters to reach requested EMIF_FREQ_C!"
                severity FAILURE;
    
	---------------------------------------------------------------------------
	-- CLOCK GENERATION
	---------------------------------------------------------------------------
	---------------------------------------------------------------------------
	-- input buffer clock
	---------------------------------------------------------------------------
-- 	ClkIn_G: IBUFG
-- 			port map (I => BECLKOUT, O => dcm_in);
    dcm_in <= BECLKOUT;
    
-- 	ClkExt_G: IBUFG
-- 			port map (I => ClockExt, O => clock_ext);
    clock_ext <= ClockExt;
	---------------------------------------------------------------------------
	-- DCM CLKIN
	---------------------------------------------------------------------------
	DCM_IN_G: DCM
	generic map (
   	    CLKIN_PERIOD 		=> CLKIN_FREQ_C,
    	DFS_FREQUENCY_MODE 	=> "LOW",
	    DLL_FREQUENCY_MODE 	=> "LOW",
    	DSS_MODE 			=> "NONE",
    	DUTY_CYCLE_CORRECTION => true,
    	PHASE_SHIFT 		=> 0,
    	CLKIN_DIVIDE_BY_2 	=> false,
    	CLKOUT_PHASE_SHIFT 	=> "NONE",
	   	CLKFX_DIVIDE 		=> CLKIN_DIV_C,
    	CLKFX_MULTIPLY 		=> CLKIN_MULT_C,
    	CLK_FEEDBACK 		=> "1X"
    )
	port map (
	    CLK0 		=> dcm_in_clk0,
	    CLK180 		=> OPEN,
	    CLK270 		=> OPEN,
	    CLK2X 		=> OPEN,
	    CLK2X180 	=> OPEN,
	    CLK90 		=> OPEN,
	    CLKDV 		=> OPEN,
	    CLKFX 		=> dcm_in_clkfx,
	    CLKFX180 	=> OPEN,
	    LOCKED 		=> dcm_in_locked,
	    PSDONE 		=> OPEN,
	    STATUS 		=> OPEN,

	    CLKFB 		=> dcm_in_clk0,
	    CLKIN 		=> clock_ext,
	    DSSEN 		=> '0',
	    PSCLK 		=> '0',
	    PSEN 		=> '0',
	    PSINCDEC 	=> '0',
	    RST 		=> dcm_in_reset
 	);

    dcm_in_reset <= '0';
    
	---------------------------------------------------------------------------
	-- DCM BECLKOUT
	---------------------------------------------------------------------------
	DCM_G: DCM
	generic map (
   	    CLKIN_PERIOD 		=> EMIF_FREQ_C,
    	DFS_FREQUENCY_MODE 	=> "LOW",
	    DLL_FREQUENCY_MODE 	=> "LOW",
    	DSS_MODE 			=> "NONE",
    	DUTY_CYCLE_CORRECTION => true,
    	PHASE_SHIFT 		=> 0,
    	CLKIN_DIVIDE_BY_2 	=> false,
    	CLKOUT_PHASE_SHIFT 	=> "NONE",
	   	CLKFX_DIVIDE 		=> DCM_DIVIDE_C,
    	CLKFX_MULTIPLY 		=> DCM_MULTIPLY_C,
    	CLK_FEEDBACK 		=> "1X"
    )
	port map (
	    CLK0 		=> dcm_clk0,
	    CLK180 		=> OPEN,
	    CLK270 		=> OPEN,
	    CLK2X 		=> OPEN,
	    CLK2X180 	=> OPEN,
	    CLK90 		=> OPEN,
	    CLKDV 		=> OPEN,
	    CLKFX 		=> dcm_clkfx,
	    CLKFX180 	=> OPEN,
	    LOCKED 		=> dcm_locked,
	    PSDONE 		=> OPEN,
	    STATUS 		=> OPEN,

	    CLKFB 		=> clock_in,
	    CLKIN 		=> dcm_in,
	    DSSEN 		=> '0',
	    PSCLK 		=> '0',
	    PSEN 		=> '0',
	    PSINCDEC 	=> '0',
	    RST 		=> dcm_reset
 	);

	---------------------------------------------------------------------------
	-- output clock buffer
	---------------------------------------------------------------------------
-- 	Clk0_G: BUFG
-- 			port map (I => dcm_clk0, O => clock_emif);

-- 	ClockMux_G: BUFGMUX
--     	port map (
-- 	      	I0 => clock_ext,
-- 	      	I1 => dcm_clk0,
-- 	      	S  => sel_beclkout,
-- 	      	O  => clock_in
-- 	    );

 		clk_ext <= clock_ext;
 		clk_dcm <= dcm_clk0;
 		clk_sel <= sel_beclkout;

	ClkFx_G: BUFG
			port map (I => dcm_clkfx, O => ClockFx);

	sel_beclkout <= '1' when dcm_reset = '0' else '0';

	---------------------------------------------------------------------------
	-- Input address
	AddrIn_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			addr_in <= (others => '0');
		elsif clock_in'event AND clock_in = '1' then
			addr_in <= unsigned(BEA);
		end if;
	end process AddrIn_P;

	---------------------------------------------------------------------------
	-- Control signals of EMIFB
	Control_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			baoe_in		<= '1';
			bawe_in		<= '1';
			bce0_in		<= '1';
			bce1_in		<= '1';
		elsif clock_in'event AND clock_in = '1' then
			baoe_in		<= BAOE;
			bawe_in		<= BAWE;
			bce0_in		<= BCE(0);
			bce1_in		<= BCE(1);
		end if;
	end process Control_P;

	---------------------------------------------------------------------------
	-- Read signal
	ReadIn_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			read_in <= '0';
		elsif clock_in'event AND clock_in = '1' then
			if BCE(2) = '0' AND BARE = '0' then
				read_in <= '1';
			else
				read_in <= '0';
			end if;
			read_data_valid <= read_in;
		end if;
	end process ReadIn_P;

	---------------------------------------------------------------------------
	-- Write signal
	WriteIn_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			write_in <= '0';
		elsif clock_in'event AND clock_in = '1' then
			if BCE(2) = '0' AND BAWE = '0' then
				write_in <= '1';
			else
				write_in <= '0';
			end if;
		end if;
	end process WriteIn_P;

	---------------------------------------------------------------------------
	-- Connection to internal DSP bus
	DspBusIn_P: process (write_in, read_in, addr_in, data_in, DspBusArr, boot_code)
	variable t : unsigned(DSP_DATA_WIDTH_C-1 downto 0);
	begin
		t := (others => '0');
		for i in 0 to N_DEVICES_C-1 loop
			DspBusArr(i).Write 	<= write_in;
			DspBusArr(i).Read	<= read_in;
			DspBusArr(i).Addr	<= addr_in;
			DspBusArr(i).DataIn	<= data_in;
-- 			DspBusArr(i).DataOut<= (others => 'Z');
			t := t OR DspBusArr(i).DataOut;
		end loop;
		data_out <= t OR ( to_unsigned(0, DSP_DATA_WIDTH_C-boot_code'length) & boot_code);
	end process DspBusIn_P;

    ---------------------------------------------------------------------------
    -- IOB instances for BED port
    ---------------------------------------------------------------------------
    BED_G : for i in 0 to DSP_DATA_WIDTH_C-1 generate
--         IOBUF_G : IOBUF
--             generic map (
--                 DRIVE      => 12,
--                 IOSTANDARD => "LVTTL",
--                 SLEW       => "FAST")
--             port map (
--                 O  => bed_in(i),
--                 IO => BED(i),
--                 I  => bed_out(i),
--                 T  => bed_tri(i));

        bed_in(i) <= BED_io_in(i);
        BED_io_out(i) <= bed_out(i);
        BED_io_tri(i) <= bed_tri(i);
        
        
        -----------------------------------------------------------------------
        -- Input data strobe process
        DataIn_P : process (areset_in, clock_in)
        begin
            if areset_in = '1' then
                data_in(i) <= '0';
            elsif clock_in'event and clock_in = '1' then
                data_in(i) <= bed_in(i);
            end if;
        end process DataIn_P;

        -----------------------------------------------------------------------
        -- Drive BED bus and output data
        DataOut_P : process (areset_in, clock_in)
        begin
            if areset_in = '1' then
                bed_tri(i) <= '1';
                bed_out(i) <= '1';
            elsif clock_in'event and clock_in = '1' then
                if read_data_valid = '1' or bce1_in = '0' then
                    bed_tri(i) <= '0';
                else
                    bed_tri(i) <= '1';
                end if;

                bed_out(i) <= data_out(i);
            end if;
        end process DataOut_P;

    end generate BED_G;
    
	---------------------------------------------------------------------------
	-- Control register
	CtrlReg_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			ctrl(CTRL_DRESETi) 	<= '0';
			ctrl(CTRL_NMIi)		<= '0';
			ctrl(CTRL_LEDi)		<= '0';
		elsif clock_in'event AND clock_in = '1' then
			if bce0_in = '0' AND bawe_in = '0' AND addr_in = X"00" then
				ctrl(CTRL_DRESETi)	<= data_in(CTRL_DRESETi);
				ctrl(CTRL_NMIi)		<= data_in(CTRL_NMIi);
				ctrl(CTRL_LEDi)		<= data_in(CTRL_LEDi);
			else
				ctrl(CTRL_DRESETi) 	<= '0';
				ctrl(CTRL_NMIi)		<= '0';
			end if;
		end if;
	end process CtrlReg_P;

	---------------------------------------------------------------------------
	-- DRESET signal generation
	DResetCnt_P: process (nreset_in, clock_in)
	begin
		if nreset_in = '0' then
        	dreset_cnt <= 0;
			dreset_done <= '0';
		elsif clock_in'event AND clock_in = '1' then
			if ctrl(CTRL_DRESETi) = '1' or dcm_in_locked = '0' then
				dreset_cnt <= 0;
				dreset_done <= '0';
			elsif dreset_done = '0' then
				dreset_cnt <= dreset_cnt + 1;
				if dreset_cnt = DRESET_PULSE_LEN_C-1 then
					dreset_done <= '1';
				else
					dreset_done <= '0';
				end if;
			end if;
		end if;
	end process DResetCnt_P;

	---------------------------------------------------------------------------
	-- NMI signal generation
	NmiCnt_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			nmi_done 	<= '1';
			nmi_cnt 	<= 0;
		elsif clock_in'event AND clock_in = '1' then
			if ctrl(CTRL_NMIi) = '1' then
				nmi_cnt 	<= NMI_PULSE_LEN_C;
				nmi_done	<= '0';
			elsif nmi_done = '0' then
				nmi_cnt <= nmi_cnt - 1;
				if nmi_cnt = 1 then
					nmi_done <= '1';
				else
					nmi_done <= '0';
				end if;
			end if;
		end if;
	end process NmiCnt_P;


	---------------------------------------------------------------------------
	-- Load basic miniprogram after DSP reset
	BootLoad_P: process (clock_in)
		variable word	: natural range 0 to BOOT_CODE_LEN_C-1;
		variable code	: unsigned(31 downto 0);
	begin
		if clock_in'event AND clock_in = '1' then
			if baoe_in = '0' AND bce1_in = '0' then
				word := to_integer( addr_in(8 downto 3) );
				code := BOOT_CODE_C( word );
				case addr_in(2 downto 1) is
               		when "00" =>	boot_code 	<= code(7 downto 0);
               		when "01" =>	boot_code 	<= code(15 downto 8);
               		when "10" =>	boot_code 	<= code(23 downto 16);
               		when others =>	boot_code 	<= code(31 downto 24);
               	end case;
			else
	        	boot_code	<= (others => '0');
			end if;
		end if;
	end process BootLoad_P;

	---------------------------------------------------------------------------
	-- Startup done pulse generator 
	--  starts when DSP write to addr 0xFF on space 1 (next  instruction is IDLE)
	--  from minibootloader
	--  startup_drive signal is active during pulse and hold time
	StartUp_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
        	startup_pulse 	<= '0';
			startup_seq 	<= '0';
			startup_drive 	<= '0';
			startup_cnt  	<= 0;
		elsif clock_in'event AND clock_in = '1' then
			-- sequency counter
		 	if bce0_in = '0' AND bawe_in = '0' AND addr_in = X"F0" then
				startup_seq <= '1';
				startup_cnt <= 0;
			elsif startup_seq = '1' then
				if startup_cnt = STARTUP_COUNT_C-1 then
					startup_seq <= '0';
				else
					startup_cnt <= startup_cnt + 1;
				end if;
			end if;
			-- pulse
			if startup_seq = '0' then
				startup_pulse 	<= '0';
				startup_drive 	<= '0';
			elsif startup_cnt = STARTUP_SETUP_C-1 then
				startup_pulse 	<= '1';
				startup_drive 	<= '1';
			elsif startup_cnt = STARTUP_COUNT_C-STARTUP_HOLD_C-1 then
				startup_pulse 	<= '0';
			elsif startup_cnt = STARTUP_COUNT_C-1 then
				startup_drive 	<= '0';
			end if;
			
		end if;
	end process StartUp_P;
	
	
	---------------------------------------------------------------------------
	-- Delay of RDY signal after FPGA's startup/reset
	RdyDelay_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
        	rdy_delay <= (others => '0');
		elsif clock_in'event AND clock_in = '1' then
			rdy_delay <= '1' & rdy_delay(0 to RDY_DELAY_C-1);
		end if;
	end process RdyDelay_P;
	
	---------------------------------------------------------------------------
	--
	AResetOutDel_P: process (areset_in, clock_in)
	begin
		if areset_in = '1' then
			arst_delay <= (others => '1');
		elsif clock_in'event AND clock_in = '1' then
			arst_delay <= '0' & arst_delay(0 to ARST_DELAY_C-1);
		end if;
	end process AResetOutDel_P;


	---------------------------------------------------------------------------
	-- GLOBAL ASSIGNMENT
	---------------------------------------------------------------------------
	nreset_in 	<= '0' when NReset_io_in = '0' and startup_seq = '0' else '1';
	nreset_out 	<= '0' when startup_pulse = '1' else '1';
	
-- 	NReset 		<= nreset_out when startup_drive = '1' else 'Z';
 	NReset_io_out <= nreset_out;
	NReset_io_tri <= not startup_drive; -- when T is 1 output is Z 
	
	AReset		<= arst_delay(ARST_DELAY_C);
-- 	ClockInt	<= clock_in;
 	clock_in <= ClockInt;

	NDReset		<= '0' when dreset_done = '0' else '1';
	NMI			<= '0' when nmi_done = '0' else '1';
	BECLKIN		<= dcm_in_clkfx;

	areset_in 	<= '1' when dreset_done = '0' OR 
							nreset_in = '0' OR 
							dcm_locked = '0' or dcm_in_locked = '0' else '0';
	dcm_reset	<= '1' when dreset_done = '0' else '0';

	BARDY 		<= '0' when rdy_delay(RDY_DELAY_C) = '0' else '1';
-- 	BBE 		<= (others => 'Z');


Debug(0) <= sel_beclkout;
Debug(1) <= dreset_done;
Debug(2) <= dcm_locked;
Debug(3) <= dcm_in_locked;
-- Debug(4) <= NReset;
Debug(5) <= startup_seq;
Debug(6) <= startup_pulse;

end RTL;
