-------------------------------------------------------------------------------
-- Entity:	Timer
--
-- Project:	Uni1p
--
-- Author:	Antonin Hegar
--
-- Description:
--	Timer for testing performance of the DSP on DX64.
--	Resolution of the timer is set by generic parameter TIMER_RES_C.
--	(Based on EmifB.vhd from Uni1P project.)
--
-- Revision:
--	$Log: Timer.vhd,v $
--	Revision 1.2  2006/04/12 05:25:51  Tony
--	Timer advance:
--	- auto repeat mode / one cycle mode
--	- generate output signal (for timer interrupt)
--	
--	Revision 1.1  2005/10/23 20:56:28  Tony
--	Initial release.
--	
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- LIBRARY
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.DX64Header.all;

-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
entity Timer is
    generic (
        BASE_ADDR_C     : natural := 32;    -- base internal address 
        FREE_RUN_C      : boolean := true; -- free run counter
        TIMER_FREQ_C    : natural := 1e6);  -- timer frequency
    port (
		-----------------------------------------------------------------------
		-- global signals
		AReset			: in std_logic;
		Clock			: in std_logic;

        -----------------------------------------------------------------------
		-- DSP interface
		DspBus			: inout DspBus_T;
        DspInt          : out std_logic
	);
end entity Timer;


-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
architecture RTL of Timer is

	---------------------------------------------------------------------------
	-- Type definition and constants
	---------------------------------------------------------------------------

	---------------------------------------------------------------------------
	-- timer predivider
-- Evaluate to real isn't still suported by ISE!!!
--    constant TIMER_PREDIV_C	: natural := natural( real(FPGA_CLOCK_FREQ_C) * TIMER_RES_C );
    constant TIMER_PREDIV_C	: natural := FPGA_CLOCK_FREQ_C / TIMER_FREQ_C;
    -- timer width (includes overflow flag)
    constant TIMER_WIDTH_C	: natural := 2*DSP_DATA_WIDTH_C;

    signal timer_tick		: std_logic;
    signal timer_reset		: std_logic;
    signal timer_prediv		: natural range 0 to TIMER_PREDIV_C-1;
	signal timer			: unsigned(TIMER_WIDTH_C-1 downto 0);
    signal timer_enable     : std_logic;

    ---------------------------------------------------------------------------
    -- Generator's signals
    signal timer_gen_reg    : unsigned(TIMER_WIDTH_C-1 downto 0) := (others => '0');
    signal timer_gen        : unsigned(TIMER_WIDTH_C-1 downto 0) := (others => '0');
    signal timer_int        : std_logic := '0';
    signal timer_gen_enable : std_logic := '0';
    signal timer_gen_set    : std_logic := '0';    
    
	---------------------------------------------------------------------------
	-- DSP bus
 	constant TIMER_LOW_ADDR_C	    : natural := BASE_ADDR_C+0;
	constant TIMER_HIGH_ADDR_C	    : natural := BASE_ADDR_C+1;
 	constant TIMER_GEN_LOW_ADDR_C	: natural := BASE_ADDR_C+2;
	constant TIMER_GEN_HIGH_ADDR_C	: natural := BASE_ADDR_C+3;
	signal sel_timer_low	        : std_logic;
	signal sel_timer_high	        : std_logic;
	signal sel_timer_gen_low	    : std_logic;
	signal sel_timer_gen_high	    : std_logic;

begin

    ---------------------------------------------------------------------------
    -- Timer counter
    ---------------------------------------------------------------------------
    Timer_P: process (AReset, Clock)
	begin
		if AReset = '1' then
			timer_prediv	<= 0;
			timer			<= (others => '0');
			timer_gen	    <= (others => '0');
            timer_gen_reg   <= (others => '0');
            timer_gen_enable<= '0';
            timer_gen_set   <= '1';
			timer_tick		<= '1';
		elsif Clock'event AND Clock = '1' then
            -------------------------------------------------------------------
            -- timer predivider
            if TIMER_PREDIV_C > 1 then
                if timer_reset = '1' then
                    timer_prediv <= 0;
                    timer_tick   <= '0';
                elsif timer_prediv = TIMER_PREDIV_C-1 then
                    timer_prediv <= 0;
                    timer_tick   <= '1';
                else
                    timer_prediv <= timer_prediv + 1;
                    timer_tick   <= '0';
                end if;
            else
                timer_tick  <= '1';
            end if;

            -------------------------------------------------------------------
            -- timer counter
			if timer_reset = '1' then
				timer <= (others => '0');
			elsif timer_tick = '1' AND timer(timer'high) = '0' and (timer_enable = '1' or FREE_RUN_C) Then
				timer <= timer + 1;
			end if;

            -------------------------------------------------------------------
            -- timer generator register
            if (not FREE_RUN_C) and timer_int = '1' then
                timer_gen_reg <= (others => '0');
                timer_gen_set <= '1';
            elsif sel_timer_gen_low = '1' and DspBus.Write = '1' then
                timer_gen_reg(DSP_DATA_WIDTH_C-1 downto 0) <= DspBus.DataIn;
                timer_gen_set <= '1';
            elsif sel_timer_gen_high = '1' and DspBus.Write = '1' then 
                timer_gen_reg(TIMER_WIDTH_C-1 downto DSP_DATA_WIDTH_C) <= DspBus.DataIn;
                timer_gen_set <= '1';
            else
                timer_gen_set <= '0';
            end if;

            if timer_gen_set = '1' or timer_gen_reg = 0 or (timer_int = '1' and (not FREE_RUN_C)) then
                timer_gen_enable <= '0';
            else
                timer_gen_enable <= '1';
            end if;

            -------------------------------------------------------------------
            -- timer generator
            if timer_gen_enable = '0' then
                timer_gen <= timer_gen_reg;
                timer_int <= '0';
            elsif timer_tick = '1' then
                if timer_gen = 1 then
                    timer_gen <= timer_gen_reg;
                    timer_int <= '1';
                else
                    timer_gen <= timer_gen - 1;
                    timer_int <= '0';
                end if;
            end if;
            
		end if;
	end process Timer_P;

    ---------------------------------------------------------------------------
    -- Timer enable flag
    -- if FREE_RUN_C is true then this flag is ignored
    ---------------------------------------------------------------------------
    Enable_P: process (AReset, Clock)
    begin
        if AReset = '1' then
            timer_enable <= '0';
        elsif Clock'event AND Clock = '1' then
            if timer_reset = '1'  then
                timer_enable <= '1';
            elsif sel_timer_low = '1' and DspBus.Read = '1' then
                timer_enable <= '0';
            end if;
        end if;
    end process Enable_P;

    ---------------------------------------------------------------------------
    -- DataOut to the DSP
    ---------------------------------------------------------------------------
  	DataOut_P: process (Clock)
  	begin
  		if Clock'event AND Clock = '1' then
  			-- sync. reset
  			if DspBus.Read = '0' OR (sel_timer_low = '0' AND sel_timer_high = '0') then
  				DspBus.DataOut <= (others => '0');
  			-- read actual timer value - low bits
  			elsif sel_timer_low = '1' then
  				DspBus.DataOut <= timer(DSP_DATA_WIDTH_C-1 downto 0);
  			-- read actual timer value - high bits
  			else
  				DspBus.DataOut <= timer(timer'high downto DSP_DATA_WIDTH_C);
  			end if;
  		end if;
  	end process DataOut_P;

    ---------------------------------------------------------------------------
    -- Address decoder
    ---------------------------------------------------------------------------
	sel_timer_low	    <= '1' when DspBus.Addr = TIMER_LOW_ADDR_C else '0';
	sel_timer_high	    <= '1' when DspBus.Addr = TIMER_HIGH_ADDR_C else '0';
	sel_timer_gen_low	<= '1' when DspBus.Addr = TIMER_GEN_LOW_ADDR_C else '0';
	sel_timer_gen_high	<= '1' when DspBus.Addr = TIMER_GEN_HIGH_ADDR_C else '0';

	------------------------------------------------------------------------------------------
	-- Timer reset 
	------------------------------------------------------------------------------------------
	timer_reset <= '1' when sel_timer_high = '1' AND DspBus.Write = '1' else '0';
    DspInt      <= timer_int;

end RTL;
